pub enum NoAlgorithms {}Expand description
Type which an implementation of Cal can use when it implements no algorithm for a
particular provider.
This type is uninhabited and can stand in for all of the Algorithm associated types as well
as state and result types.