pub struct EmptyCal;Expand description
An implementation of Cal that provides no single algorithm.
It implements all the individual traits, as well as the full Cal trait. The former is useful
for hardware implementations that don’t touch an area at all; the latter is useful in testing
or when an extender is used standalone.